#!/usr/bin/perl
#
# Copyright (c) 2010-2016 Wanos Networks Pty(Ltd). All rights reserved.
#
# Modification of this software is not permitted under the licensing
# agreement. Please read the license as well as terms and conditions at
# http://wanos.org/license or email info@wanos.org for more information.
#


my ($mac, $cpu) = @ARGV;

open(CONFIG, "<" , "/tce/etc/wanos/wanos.conf") || die("Couldn't open the file. Error: $!\n");
    while (<CONFIG>)
    {
    chomp;
    next if /^\s*\#/;
    next unless /=/;
    my ($key, $variable) = split(/\s*=\s*/,$_,2);
    $variable =~ s/(\$(\w+))/$config{$2}/g;
        $config{$key} = $variable;
    }
close CONFIG;

my %virtual_nets = ();
my $virt_lan0;
my $virt_mask;
my $exit_last;
my $deployment_mode = $config{DEPLOYMENT_MODE};
my $ipno = qr/2(?:5[0-5]|[0-4]\d)|1\d\d|[1-9]?\d/x;

my $network, $mask, $gw;
my @route_list;
if ( open(ROUTES, "<" , "/tce/etc/wanos/static_routes") )
{
    while (<ROUTES>)
    {
    chomp;
    next if /^\s*\#/;
    next unless /\//;
    next if /^\s*$/;
    my ($network, $variable) = split(/\//,$_,2);
    my ($mask, $gw) = split(/ /,$variable,2);
    $mask =~ s/^\s+|\s+$//g;
    $gw =~ s/^\s+|\s+$//g;
    if ( ($network !~ /^($ipno\.){3}$ipno$/) || ($gw !~ /^($ipno\.){3}$ipno$/) || ($mask < 8) || ($mask > 32) )
        {
        print STDERR "Error : Invalid IP route configured in static_routes \n";
        last;
        }
    push @route_list, $_;
    }
}
else { print STDERR "Error : Can't open static routes file. Continue without optional routes\n"; }
close ROUTES;




if ($deployment_mode eq "router" && $config{VIRT_LAN0} ne '')
{
	($virt_lan0, $virt_mask) = split('/',$config{VIRT_LAN0});
	if (($virt_lan0 =~ /^($ipno\.){3}$ipno$/ ) && ($virt_mask >= 7) && ($virt_mask <= 32))
	{
	 $virtual_nets{$virt_lan0} = $virt_mask; 
	 $exit_last = $virt_lan0;
	}
	else
	{
	 print STDERR "Error : VIRT_LAN0 Address / Mask validation failed: $config{VIRT_LAN0} \n";
	 $deployment_mode = "bridge";
	}

	open(ROW, "<" , "/tce/etc/wanos/virtual_lan") || die("Couldn't open the file. Error: $!\n");
	while (<ROW>)
	{
		chomp;
		next if /^\s*$/;
		next if /^\s*\#/;
		($virt_lan0, $virt_mask) = split('/',$_);
		if (($virt_lan0 =~ /^($ipno\.){3}$ipno$/ ) && ($virt_mask >= 7) && ($virt_mask <= 32)) { $virtual_nets{$virt_lan0} = $virt_mask; }
		else { print STDERR "Warn : virtual_lan list Address / Mask validation failed: " . $_ . " \n"; }
	}
	close ROW;
}


$matchmac=$mac;
$matchmac=~tr/://d;

my $encapproto = "6C";
my $proto_num = 108;
if ($config{ENCAPPROTO} eq "gre")
    {
    $encapproto = "2F";
    $proto_num = 47;
    }


print <<EOF;

//
// Deployment method
//

EOF

if ($deployment_mode eq "bridge")
{
	if ($config{PMETHOD} eq "NETMAP") { print "wan0 :: FromDevice(netmap:wan0"; }
	else { print "wan0 :: FromDevice(wan0"; }
	print ", SNIFFER false, PROMISC true, METHOD $config{PMETHOD});\n";
	if ($config{PMETHOD} eq "NETMAP") { print "lan0 :: FromDevice(netmap:lan0"; }
	else { print "lan0 :: FromDevice(lan0"; }
	print ", SNIFFER false, PROMISC true, METHOD $config{PMETHOD});\n";
	if ($config{PMETHOD} eq "NETMAP") { print "lan0out :: VLANEncap(ANNO) -> ToDevice(netmap:lan0, METHOD $config{PMETHOD});\n"; }
	else { print "lan0out :: VLANEncap(ANNO) -> ToDevice(lan0, METHOD $config{PMETHOD});\n"; }
	if ($config{PMETHOD} eq "NETMAP") { print "tdwan0 :: VLANEncap(ANNO) -> ToDevice(netmap:wan0, METHOD $config{PMETHOD});\n"; }
	else { print "tdwan0 :: VLANEncap(ANNO) -> ToDevice(wan0, METHOD $config{PMETHOD});\n"; }
	print "Idle -> [0]lanarp -> Discard;\n";
}
elsif ($deployment_mode eq "router")
{

print "AddressInfo(virtualwan $config{IP} $mac";

my $i = 0;
foreach (keys %virtual_nets)
	{ 
	print " , virtuallan$i $_  00:00:CC:CC:CC:CC "; 
	$i++;
	}
print ");\n";

print <<EOF;
lan0 :: Null;
wan0 :: Null;
fd0 :: FromDevice(wan0, SNIFFER false, PROMISC true, METHOD PCAP);
wan0out :: ThreadSafeQueue(1000) -> ToDevice(wan0, METHOD PCAP);
etherencap :: Classifier(12/0800, -) -> Strip(14) -> virtual_encap_check::CheckIPHeader(VERBOSE $config{VERBOSE}) -> RadixIPLookup($config{NET}/$config{MASK}  0, 
EOF
;

foreach (keys %virtual_nets) { print " $_/$virtual_nets{$_} 0 , "; }

print <<EOF;
0.0.0.0/0 $config{GW} 0) -> [0]lanarp -> wan0out;
etherencap[1] -> wan0out;
lan0out :: Unqueue -> etherencap;
tdwan0 :: Unqueue -> etherencap;
fd0[0] -> virtual_split_filter :: Classifier(0/$matchmac 12/0800, 0/0000CCCCCCCC 12/0800, 12/0806 20/0001, 12/0806 20/0002);
virtual_split_filter[0] -> Strip(14) -> virtual_wan_check::CheckIPHeader(VERBOSE $config{VERBOSE}) -> DecIPTTL -> virtual_wanclass::IPClassifier(dst $config{IP}, -);
virtual_split_filter[1] -> Strip(14) -> virtual_lan_check::CheckIPHeader(VERBOSE $config{VERBOSE}) -> DecIPTTL -> virtual_lanclass::IPClassifier(dst $config{IP}, 
EOF
;

foreach (keys %virtual_nets)
	{
	print " dst $_  ";
	print " or " if !/$exit_last/;
	}
print " , \n";

foreach (keys %virtual_nets)
	{
	print " dst $_/$virtual_nets{$_}  ";
	print " or " if !/$exit_last/;
	}
print " , -);\n";
print "virtual_split_filter[2] -> ARPResponder(virtualwan ";
$i=0;
foreach (keys %virtual_nets) { print ", virtuallan$i "; $i++; }
print ") -> wan0out;\n";
print <<EOF;
virtual_split_filter[3] -> [1]lanarp; 
virtual_wanclass[0] -> Unstrip(14) -> wan0;
virtual_wanclass[1] -> EtherEncap(0x0800, 1:1:1:1:1:1, 2:2:2:2:2:2) -> wan0;
virtual_lanclass[0] -> Unstrip(14) -> lan0;
virtual_lanclass[1] -> ICMPPingResponder -> [0]lanarp;
virtual_lanclass[2] -> [0]lanarp;
virtual_lanclass[3] -> EtherEncap(0x0800, 2:2:2:2:2:2, 1:1:1:1:1:1) -> lan0;
EOF
;

}
elsif ($deployment_mode eq "tunnel")
{
print <<EOF

AddressInfo(wan0 $config{IP} $mac);
lan0 :: Null;
wan0 :: Null;
fd0 :: FromDevice(wan0, SNIFFER false, PROMISC true, METHOD PCAP);
wan0out :: ThreadSafeQueue(1000) -> ToDevice(wan0, METHOD PCAP);

etherencap :: Classifier(12/0800, -) -> Strip(14) -> encap_ipcheck::CheckIPHeader(VERBOSE true) -> RadixIPLookup(
EOF
;

if (scalar @route_list > 0) { foreach (@route_list) { print $_ . " 0, \n"; } }

print <<EOF
$config{NET}/$config{MASK}  0, 0.0.0.0/0 $config{GW} 0) -> [0]lanarp -> wan0out;
etherencap[1] -> wan0out;

lan0out :: Unqueue -> etherencap;
tdwan0 :: Unqueue -> etherencap;

fd0[0] -> l2_filter :: Classifier(0/$matchmac 12/0800, 12/0806 20/0001, 12/0806 20/0002);
l2_filter[0] -> Strip(14) -> wan_check::CheckIPHeader(VERBOSE true) -> tunnel_drop_broadbast::DropBroadcasts -> decttl::DecIPTTL -> host_filter::IPClassifier( dst $config{IP}, -);

l2_filter[1] -> ARPResponder(wan0) -> wan0out;
l2_filter[2] -> [1]lanarp;

split_filter::IPClassifier( (dst $config{IP} and src port 4050 and dst port 4050) or (dst $config{IP} and proto 108), -);
split_filter[0] -> UDPIPDecap -> check_len::CheckLength(1500) -> to_wan_etherencap::EtherEncap(0x0800, 1:1:1:1:1:1, 2:2:2:2:2:2) -> wan0;
split_filter[1] -> to_host_frag::WanosIPFragmenter(1500) -> Unstrip(14) -> lan0;

host_filter[0] -> tunnel_reassmbler::IPReassembler -> split_filter;
host_filter[1] -> to_lan_ethercap::EtherEncap(0x0800, 2:2:2:2:2:2, 1:1:1:1:1:1) -> lan0;

decttl[1]    -> ICMPError($config{IP}, 11, 0) -> encap_ipcheck;
check_len[1] -> ICMPError($config{IP}, unreachable, needfrag) -> encap_ipcheck;

EOF
;

}
else { print STDERR "Error : unknown deployment option\n"; }




