#!/usr/local/bin/bash

test ! `dmesg | grep -ic amazon` -eq 1 && exit

mask2cidr() {
    nbits=0
    IFS=.
    for dec in $1 ; do
        case $dec in
            255) let nbits+=8;;
            254) let nbits+=7;;
            252) let nbits+=6;;
            248) let nbits+=5;;
            240) let nbits+=4;;
            224) let nbits+=3;;
            192) let nbits+=2;;
            128) let nbits+=1;;
            0);;
            *) echo "Error: $dec is not recognised"; exit 1
        esac
    done
    echo "$nbits"
}

function valid_ip()
{
    local  ip=$1
    local  stat=1

    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
            && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}


IP=`/sbin/ifconfig eth0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'`
GW=`route -n | grep 'G[ \t]' | awk '{print $2}'`
NET=""
MASK=`/sbin/ifconfig eth0 | awk -F: '/Mask:/{print $4}'`

IFS=. read -r i1 i2 i3 i4 <<< $IP
IFS=. read -r m1 m2 m3 m4 <<< $MASK
NET=`printf "%d.%d.%d.%d\n" "$((i1 & m1))" "$((i2 & m2))" "$((i3 & m3))" "$((i4 & m4))"`

if valid_ip $IP; then stat='good'; else exit 1; fi
#        printf "%-20s: %s\n" "$IP" "$stat"
if valid_ip $GW; then stat='good'; else exit 1; fi
#        printf "%-20s: %s\n" "$GW" "$stat"
if valid_ip $MASK; then stat='good'; else exit 1; fi
#        printf "%-20s: %s\n" "$MASK" "$stat"
if valid_ip $NET; then stat='good'; else exit 1; fi
#        printf "%-20s: %s\n" "$NET" "$stat"

MASK=$(mask2cidr $MASK)

# Write AWS IP to wanos.conf
sed -i -r "s/(^IP=*).*/\1$IP/" /tce/etc/wanos/wanos.conf
sed -i -r "s/(^MASK=*).*/\1$MASK/" /tce/etc/wanos/wanos.conf
sed -i -r "s/(^NET=*).*/\1$NET/" /tce/etc/wanos/wanos.conf
sed -i -r "s/(^GW=*).*/\1$GW/" /tce/etc/wanos/wanos.conf

test ! -z "`pgrep dhclient`" && sudo killall dhclient 2> /dev/null
ip addr flush dev eth0

