#!/usr/bin/perl
#
# Copyright (c) 2010-2016 Wanos Networks Pty(Ltd). All rights reserved.
#
# Modification of this software is not permitted under the licensing
# agreement. Please read the license as well as terms and conditions at
# http://wanos.co/license or email info@wanos.co for more information.
#

use CGI;
use warnings;
use Cwd qw(abs_path);

my $cgi = CGI->new;
my $update = $cgi->param("update");

open(CONFIG, "<", "/tce/etc/wanos/wanos.conf") || die("Couldn't open the file. Error: $!\n");
while (<CONFIG>)
{
	chomp;
	next if /^\s*\#/;
	next unless /=/;
	my ($key, $variable) = split(/\s*=\s*/, $_, 2);
	$variable =~ s/(\$(\w+))/$config{$2}/g;
	$config{$key} = $variable;
}
close CONFIG;

my %sites;
open(ROW, "<", "/tce/etc/wanos/peersites") || die("Couldn't open the file. Error: $!\n");
while (<ROW>)
{
	chomp;
	next if /^\s*\#/;
	next unless /=/;
	my @values = split(/,/, $_);
	$sites{$values[1]} = 1;
}
close ROW;

my $reconf;
open(INFO, "<", "/tce/etc/wanos/reconfigure") || die("Couldn't open the file. Error: $!\n");
$reconf = <INFO>;
chomp($reconf);
close INFO;
if ($reconf ne '') {
	$reconf = "<span class='low'>No</span>";
}
else {
	$reconf = "<span class='good'>Ok</span>";
}

my $datastore = ($config{DS_SIZE} / 1000);
my $datastore_mem = ($config{DS_SIZE} / 16) * 1000;
my $cache_size = $config{HTTP_CACHE_SIZE} / 1000;
my $cache_mem = $config{HTTP_CACHE_MEM} * 1000;

my $diskerr = "<span class='good'>Ok</span>";
my $memerr = "<span class='good'>Ok</span>";
my $cputhreads = "<span class='good'>Ok</span>";
my $linkerr = "<span class='good'>Ok</span>";
my $linkdrops = "<span class='good'>Ok</span>";
my $linkduplex = "<span class='good'>Ok</span>";
my $chpasswd = "<span class='good'>Ok</span>";
my $netflow = "<span class='good'>Ok</span>";
my $wanosd = "<span class='good'>Ok</span>";

my $totalsites = scalar keys %sites;
$memsize = (split " ", `head -1 /proc/meminfo`)[1];
$memrequired = $totalsites * $datastore_mem;
if ($config{HTTP_CACHE_REDIRECT} eq 'true') {
	$memrequired = $memrequired + $cache_mem;
}
if ($memrequired >= $memsize) {
	$memerr = "<span class='bad'>Fail</span>";
}

$disksize = `df -h /wanos | tail -1`;
@disksize = split(' ', $disksize);
$disksize = $disksize[1];
if ($disksize =~ /T/) {
	$disksize = $disksize * 1024;
}
if ($disksize =~ /M/) {
	$disksize = $disksize / 1024;
}
$diskrequired = $totalsites * $datastore;
if ($config{HTTP_CACHE_REDIRECT} eq 'true') {
	$diskrequired = $diskrequired + 10;
}
if ($diskrequired >= $disksize) {
	$diskerr = "<span class='bad'>Fail</span>";
}

$cpu = `awk '/^processor/ {count++} END {print count}' /proc/cpuinfo`;
if ($cpu <= 1) {
	$cputhreads = "<span class='low'>Low</span>";
}

my @duplex = `cat /sys/class/net/*an0/duplex`;
foreach (@duplex)
{
	next if /full/;
	$linkduplex = "<span class='bad'>Fail</span>";
}
my @linkerrors = `cat /sys/class/net/*an0/statistics/rx_errors`;
foreach (@linkerrors)
{
	next if $_ < 100;
	$linkerr = "<span class='bad'>Fail</span>";
}
my @linkdrops = `cat /sys/class/net/*an0/statistics/rx_drops`;
foreach (@linkdrops)
{
	next if $_ < 100;
	$linkdrops = "<span class='bad'>Fail</span>";
}

my $pass_update = `date +%s`;
$pass_update = ( $pass_update - $config{CHPASSWD} );
if (( $config{CHPASSWD} eq '' ) || ( $pass_update > 7776000 )) {
	$chpasswd = "<span class='bad'>Fail</span>";
}

# GW wan0 check           
my $gw_mac;
my $gw_int;
my @arp_table = `wanos-show arptable.table`;
my @switch_table = `wanos-show sw.table`;
my $tmp_cfg;

foreach (@arp_table)
{
	$gw_ip = (split ' ', $_)[0];
	if ("$gw_ip" eq "$config{GW}")
	{
		$gw_mac = (split ' ', $_)[2];
		last;
	}
}

foreach (@switch_table)
{
	next if not /$gw_mac/;
	$gw_int = (split ' ', $_)[1];
	last;
}

my $gw = "<span class='bad'>Fail</span>";
if ($gw_int eq '1' || $config{DEPLOYMENT_MODE} ne "bridge" ) {
	$gw = "<span class='good'>Ok</span>";
}



# NetFlow
my $netflow_pid = `pgrep nfcapd`;
my $netflow_c = ($netflow_pid =~ tr/\n//);
if ($netflow_c ne '2') {
	$netflow = "<span class='bad'>Fail</span>";
}

# Wanosd
my $wanosd_pid = `pgrep -f wanosd`;
my $wanosd_c = ($wanosd_pid =~ tr/\n//);
if ($wanosd_c < '1') {
	$wanosd = "<span class='bad'>Fail</span>";
}


print <<END;

<div class="page-title">
    <div class="title_left">
        <h3>Health Status</h3>
    </div>
</div>
<div class="clearfix"></div>

<div class="row">
    <div class="col-md-8 col-sm-8 col-xs-12">
        <div class="x_title title_block">
            <h2>System Health Check</h2>
            <ul class="nav navbar-right panel_toolbox">
                <li><a class="collapse-link"><i class="fa fa-chevron-up"></i></a>
                </li>
                <li><a class="close-link"><i class="fa fa-close"></i></a>
                </li>
            </ul>
            <div class="clearfix"></div>
        </div>
        <div class="x_panel fixed_height_320">
            <div class="x_content">

END
;

print <<END

    <table class="table table-striped" width='100%'>
		<tr> <td title='Sufficient Diskspace Available'>Diskspace</td> <td title='Sufficient Diskspace Available'>$diskerr</td></tr>
		<tr> <td title='Sufficient RAM Memory Available'>Memory</td> <td title='Sufficient RAM Memory Available'>$memerr</td></tr>
		<tr> <td title='Minimum recommended CPU Threads: 2'>CPU </td> <td title='Minimum recommended CPU Threads: 2'>$cputhreads</td></tr>
		<tr> <td title='Ethernet Interface Duplex'>Link Duplex </td> <td title='Ethernet Interface Duplex'>$linkduplex</td></tr>
		<tr> <td title='Ethernet Interface Link Errors'>Link Errors </td> <td title='Ethernet Interface Link Errors'>$linkerr</td></tr>
		<tr> <td title='Ethernet Interface Packet Drops'>Link Drops </td> <td title='Ethernet Interface Packet Drops'>$linkdrops</td></tr>
		<tr> <td title='NetFlow processes running'>NetFlow</td> <td title='NetFlow processes running'>$netflow</td></tr>
		<tr> <td title='Wanosd Watchdog process running'>Watchdog</td> <td title='Wanosd Watchdog process running'>$wanosd</td></tr>
		<tr> <td title='Check that the gateway is found on the wan0 interface'>Wan Gateway</td> <td title='Check that the gateway is found on the wan0 interface'>$gw</td></tr>
		<tr> <td title='All pending config changes applied'>Configs Applied </td> <td title='All pending config changes applied'>$reconf</td></tr>
		<tr> <td title='Passwords changed within the last three months.'>Passwords Changed </td> <td title='Passwords changed within the last three months.'>$chpasswd</td></tr>
	</table>

END
;

print <<END

        </div>
    </div>

</div>
END
;
