#!/usr/bin/perl
#
# Copyright (c) 2010-2016 Wanos Networks Pty(Ltd). All rights reserved.
#
# Modification of this software is not permitted under the licensing
# agreement. Please read the license as well as terms and conditions at
# http://wanos.co/license or email info@wanos.co for more information.
#


use CGI;
use warnings;
use autodie;
use Scalar::Util qw(looks_like_number);

my $cgi = CGI->new;
my $policy = $cgi->param("policy");
my $dstnet = $cgi->param("dstnet");
my $sitenum = $cgi->param("sitenum");
my $sitename = $cgi->param("sitename");
my $remove = $cgi->param("remove");
my $peerip = $cgi->param("peerip");
my $update = 0;
my $paramerror = 0;
my $removed = 0;
my $errmsg = "";
my @dst = ();

my $ajax_trigger = 0;
my $ajax_trigger = $cgi->param("ajax_trigger");
my $ajax_table_trigger = 0;
my $ajax_table_trigger = $cgi->param("ajax_table_trigger");

if ($policy ne '') {
	$update = 1;
}
if (($policy < 1) || ($policy > 100 )) {
	$policy = 101;
}
if ($dstnet eq '') {
	$dstnet = "0.0.0.0/0";
}

my $disabled = "";
my $status = (split "=", `head -1 /tce/etc/wanos/license.info`)[1];
chomp($status);
if ($status eq 'Unlicensed') {
#	$disabled = "disabled";
}

my $disksize;
my $diskrequired;
my $datastore;
my $totalmem;

my @rows;
my %sites;
my $i = 0;

open(ROW, "<", "/tce/etc/wanos/peersites") || die("Couldn't open the file. Error: $!\n");
while (<ROW>)
{
	chomp;
	next if /^\s*\#/;
	next unless /=/;
	my ($key, $commastring) = split(/\s*=\s*/, $_, 2);
	$commastring =~ s/(\$(\w+))/$rows{$2}/g;
	$rows{$key} = $commastring;
	my @values = split(',', $commastring);
	$sites{$values[1]} = 1;
	$i++;
}
close ROW;
my $totalsites = scalar keys %sites;

my @config;
open(CONFIG, "<", "/tce/etc/wanos/wanos.conf") || die("Couldn't open the file. Error: $!\n");
while (<CONFIG>)
{
	chomp;
	next if /^\s*\#/;
	next unless /=/;
	my ($key, $variable) = split(/\s*=\s*/, $_, 2);
	$variable =~ s/(\$(\w+))/$config{$2}/g;
	$config{$key} = $variable;
}
close CONFIG;
$datastore = ($config{DS_SIZE} / 1000);
$datastore_mem = sprintf( "%0.f", ($config{DS_SIZE} / 17) * 1000 );
$cache_size = $config{HTTP_CACHE_SIZE} / 1000;
$cache_mem = $config{HTTP_CACHE_MEM} * 1000;

# Skip rules if the values are suspect
if ($update == 1)
{
	@dst = split(/\//, $dstnet);
	my $ipno = qr/2(?:5[0-5]|[0-4]\d)|1\d\d|[1-9]?\d/x;
	if ($dst[0] !~ /^($ipno\.){3}$ipno$/) {
		$update = 0;
		$paramerror = 2;
	}
	if (!looks_like_number($dst[1])) {
		$update = 0;
		$paramerror = 3;
	}
	if (( $dst[1] > 32) || ( $dst[1] < 0 )) {
		$update = 0;
		$paramerror = 3;
	}
	# if ($totalsites > 5) { $update = 0; $paramerror = 4;}
	$totalmem = (split " ", `head -1 /proc/meminfo`)[1];
	$memrequired = ($totalsites * $datastore_mem);
	if ($memrequired >= $totalmem) {
		$update = 0;
		$paramerror = 5;
	}
	if (( $peerip ne ``) && ( $peerip !~ /^($ipno\.){3}$ipno$/ )) {
		$update = 0;
		$paramerror = 6;
	}

	$sitename =~ s/,//g;
}

if ($update == 1)
{
	foreach my $check (keys %rows)
	{
		if ($check == $policy) {
			$update = 0;
			$paramerror = 8;
		}
	}
}

if ($update == 1)
{
	open(OUT, ">>", "/tce/etc/wanos/peersites") || die("Couldn't open the file. Error: $!\n");
	my $updatestring = "$policy" . "=" . "$dst[0]/$dst[1],$sitenum,$sitename,$peerip";
	print OUT $updatestring . "\n";
	($key, $commastring) = split(/\s*=\s*/, $updatestring, 2);
	$commastring =~ s/(\$(\w+))/$rows{$2}/g;
	$rows{$key} = $commastring;
	$i++;
	close OUT;

	open(OUT, ">", "/tce/etc/wanos/reconfigure") || die("Couldn't open the file. Error: $!\n");
	print OUT "1";
	close OUT;
}

if (($remove ne '') && (looks_like_number($remove)))
{
	open(OUT, ">", "/tce/etc/wanos/peersites") || die("Couldn't open the file. Error: $!\n");
	foreach my $num (keys %rows)
	{
		$line = $num . "=" . "$rows{$num}" . "\n";
		print OUT $line unless ( $line =~ m/^$remove=/ );
	}
	close(OUT);
	$removed = $remove;

	open(OUT, ">", "/tce/etc/wanos/reconfigure") || die("Couldn't open the file. Error: $!\n");
	print OUT "1";
	close OUT;
}

if ($ajax_trigger == 0) {

	if ($ajax_table_trigger == 0) {

		print <<END;

<div class="page-title">
    <div class="title_left">
        <h3>MultiSite</h3>
    </div>
</div>
<div class="clearfix"></div>

<div class="row">
    <div class="col-md-12 col-sm-12 col-xs-12">
        <div class="x_title title_block">
            <h2>MultiSite Policies</h2>
            <ul class="nav navbar-right panel_toolbox">
                <li><a class="collapse-link"><i class="fa fa-chevron-up"></i></a>
                </li>
                <li><a class="close-link"><i class="fa fa-close"></i></a>
                </li>
            </ul>
            <div class="clearfix"></div>
        </div>
        <div class="x_panel fixed_height_320">
            <div id="multisite" class="x_content">
END
		;

	}
}

if ($ajax_table_trigger == 2) {

		print <<END
		<div class="alert alert-success alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Done! </strong> Rule # $removed removed. Apply the new policy by restarting the Wanos service under <a target='_parent' href='#/reset'>Configure > Reset</a>
		</div>

END
;
}

if ($ajax_trigger == 0) {

	my %sites;
	open(SITES, "<", "/tce/etc/wanos/peersites") || die("Couldn't open the file. Error: $!\n");
	while (<SITES>)
	{
		chomp;
		next if /^\s*\#/;
		next unless /=/;
		my @values = split(/,/, $_);
		$sites{$values[1]} = 1;
	}
	close SITES;
	$totalsites = scalar keys %sites;

	if ($totalsites == 0) {

		print <<END
		<div class="alert alert-danger alert-dismissible fade in" role="alert">
		    <strong>Warning!</strong> No remote site subnets configured. The default subnet is required.
		</div>
END
	;

	}

	$disksize = `df -h /wanos | tail -1`;
	@disksize = split(' ', $disksize);
	$disksize = $disksize[1];
	if ($disksize =~ /T/) {
		$disksize = $disksize * 1024;
	}
	if ($disksize =~ /M/) {
		$disksize = $disksize / 1024;
	}
	$diskrequired = $totalsites * ($datastore+1);
	if ($config{HTTP_CACHE_REDIRECT} eq 'true') {
		$diskrequired = $diskrequired + $cache_size;
	}

	if ($diskrequired >= $disksize) {
		print <<END
		<div class="alert alert-danger alert-dismissible fade in" role="alert">
		    <strong>Warning!</strong> Insufficient disk space on partition ($disksize Available). Expand or add partition ($diskrequired GB Required).
		</div>
END
	;

	}

	$totalmem = (split " ", `head -1 /proc/meminfo`)[1];
	$memrequired = ($totalsites * $datastore_mem);
	if ($config{HTTP_CACHE_REDIRECT} eq 'true') {
		$memrequired = $memrequired + $cache_mem;
	}
	if ($memrequired >= $totalmem) {

		print <<END
		<div class="alert alert-danger alert-dismissible fade in" role="alert">
		    <strong>Warning!</strong> Insufficient RAM Memory ($totalmem KB Available - $memrequired KB Required).
		</div>
END
	;
	}

	print <<END
                <table id="peersites_table" class="table table-striped">
		            <thead>
                        <tr>
                            <th width='15%'>
                                Rule #
                            </th>
                            <th width='20%' title='Remote Site Network Subnets'>
                               Remote Site Subnet
                            </th>
                            <th width='15%' title='Group Multiple Subnets together'>
                               Site Number
                            </th>
                            <th width='20%' title='Optional Remote Peer IP Address&#013;For Example: 10.10.20.254'>
                               Peer IP
                            </th>
                            <th width='20%' title='Name, Comment or Description'>
                               Description
                            </th>
                            <th width='15%' title='Remove this rule'>
                               Delete
                            </th>
                        </tr>
		            </thead>
END
;

	foreach (sort {
		$a <=> $b
	} keys(%rows))
	{
		next if $_ == $removed;
		my @values = split(',', $rows{$_});
		print  "<tr data-id='$_'><td><center>$_</center></td><td title='Remote Site Network Subnet'>$values[0]</td><td title='Group Multiple Subnets together'><center>$values[1]</center></td><td title='Optional Remote Peer IP Address&#013;For Example: 10.10.20.254'>";
		if ($values[3] eq '') {
			print "Auto";
		}
		else {
			print $values[3];
		}
		print "</td><td title='Name, Comment or Description'>$values[2]</td>\n";
		if ($_ < 100) {
			print  "<td><a class='btn_trash' href='#' data-id='$_' title='Remove this rule' ><i class='fa fa-trash-o'></i></a></td></tr>\n";
		}
		elsif ($_ == 100) {
			print  "<td title='Default rule is required.' ></td></tr>\n";
		}
		else {
			print "<td> </td></tr>\n";
		}
	}
	print "</table>";

	if ($ajax_table_trigger == 0) {
		print <<END

            </div>
        </div>
    </div>
</div>
END
	;

	}
	if ($ajax_table_trigger == 0) {

		print <<EOF;

<div class="row">
    <div class="col-md-6 col-sm-6 col-xs-12">
        <div class="x_title title_block">
            <h2>Add Site</h2>
            <ul class="nav navbar-right panel_toolbox">
                <li><a class="collapse-link"><i class="fa fa-chevron-up"></i></a>
                </li>
                <li><a class="close-link"><i class="fa fa-close"></i></a>
                </li>
            </ul>
            <div class="clearfix"></div>
        </div>
        <div class="x_panel fixed_height_320">
            <div class="x_content">


EOF
		;

	}

}

$i++;

if ($ajax_table_trigger == 0) {

	if ($update == 1) {

		print <<END
		<div class="alert alert-success alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Done! </strong> Peer Site Policy updated. Apply the new policy by restarting the Wanos service under <a target='_parent' href='#/reset'>Configure > Reset</a>
		</div>

END
	;
	}
}

if ($ajax_table_trigger == 0) {
	if ($paramerror == 1) {
		$errmsg = $errmsg . " Invalid Source Network<br />\n";
	}
	if ($paramerror == 2) {
		$errmsg = $errmsg . " Invalid Destination Network<br />\n";
	}
	if ($paramerror == 3) {
		$errmsg = $errmsg . " Invalid Network Mask<br />\n";
	}
	if ($paramerror == 4) {
		$errmsg = $errmsg . " Could not add site. 32-Bit platform supports a maximum of 4 peers<br />\n";
	}
	if ($paramerror == 5) {
		$errmsg = $errmsg . " Insufficient memory for additional remote site: $totalsites remote sites requires $totalsites GB.<br />\n";
	}
	if ($paramerror == 6) {
		$errmsg = $errmsg . " Invalid Remote Peer IP Address.<br />\n";
	}
	if ($paramerror == 7) {
		$errmsg = $errmsg . " Shape requires a Rate<br />\n";
	}
	if ($paramerror == 8) {
		$errmsg = $errmsg . " Duplicate Rule Number<br />\n";
	}
	if ($paramerror == 9) {
		print $errmsg . "<br />\n";
	}

	if ($errmsg ne '') {
		print <<END
		<div class="alert alert-danger alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Warning! </strong> $errmsg
		</div>

END
	;
	}
}

if ($ajax_table_trigger == 0) {

	if ($removed > 0) {
		print <<END
		<div class="alert alert-success alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Done! </strong> Rule # $removed removed. Apply the new policy by restarting the Wanos service under <a target='_parent' href='#/reset'>Configure > Reset</a>
		</div>

END
	;
	}
}

if ($ajax_table_trigger == 0) {

	if ($removed == 100) {
		print <<END
		<div class="alert alert-danger alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Warning! </strong> Removing the default site is not recommended. Please add a default catch all site.
		</div>

END
	;
	}

}

if ($ajax_table_trigger == 0) {

	print <<END

				<form id="form_multisite" action="" class="form-horizontal form-label-left" name="addrule" action="" method="post" data-parsley-validate>
					<div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="policy">Rule #
                        </label>
                        <div class="col-md-3 col-sm-3 col-xs-12">
                            <input id="policy" name="policy" title="Rule Number (1-99)"
		                        type="text" value="$i" min="1" max="99" size="1"
		                        class="form-control col-md-3 col-lm-3 col-xs-12" required
		                        data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="dstnet">Remote Site Subnet</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="dstnet" title="Traffic Source Network & Mask"
                            type="text" size="14" maxlength="18" data-parsley-check_ip_and_mask
                            id="dstnet" class="form-control col-md-4 col-xs-12" required
                            data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="sitenum">Site Number</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <select id="sitenum" name="sitenum" title="Group Multiple Subnets together" class="form-control" data-toggle="tooltip">
END
;

	print "<option value='0'>0</option>\n";
	for ($c = 1; $c <= 64; $c++) {
		print "<option value='$c' $disabled>$c</option>\n";
	}

	print <<END;
</select>
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="peerip">Peer IP <span class="description">For Example: 10.10.20.254</span></label>
                         <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="peerip" title="Optional Remote Peer IP Address&#013;For Example: 10.10.20.254"
                            size="20" maxlength="15" type="text" id="peerip"
                            class="form-control col-md-4 col-xs-12" 
                            data-parsley-check_ip
                            data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="sitename">Description</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="sitename" type="text" title="String description of sitename"
                            type="text" size="20" maxlength="40" id="sitename"
                            class="form-control col-md-4 col-xs-12" required
                            data-toggle="tooltip" />
                        </div>
                    </div>


                    <div class="ln_solid"></div>
                    <div class="form-group">
                        <div class="col-md-6 col-sm-6 col-xs-12 col-md-offset-3">
                            <a type="button" class="btn btn-default" href="/#/system_settings" onclick="window.location.reload();return false;" style='font-size: 8pt;'>Cancel</a>
                            <button id="btn_submit" type="submit" class="btn btn-success" style='font-size: 8pt;'>Submit</button>
                        </div>
                    </div>
                </form>
END
	;

	if ($ajax_trigger == 0) {
		print <<END
			</div>
		</div>
	</div>
</div>

END
	;
	}
}
