#!/usr/bin/perl
#
# Copyright (c) 2010-2016 Wanos Networks Pty(Ltd). All rights reserved.
#
# Modification of this software is not permitted under the licensing
# agreement. Please read the license as well as terms and conditions at
# http://wanos.co/license or email info@wanos.co for more information.
#

use CGI;
use warnings;
use autodie;
use Scalar::Util qw(looks_like_number);

my $cgi = CGI->new;
my $policy = $cgi->param("policy");
my $srcnet = $cgi->param("srcnet");
my $dstnet = $cgi->param("dstnet");
my $proto = $cgi->param("proto");
my $port = $cgi->param("port");
my $dscp = $cgi->param("dscp");
my $class = $cgi->param("class");
my $action = $cgi->param("action");
my $rate = $cgi->param("rate");
my $bypass = $cgi->param("bypass");
my $remove = $cgi->param("remove");
my $edit = $cgi->param("edit");
my $removeedit = $cgi->param("removeedit");

my $update = 0;
my $paramerror = 0;
my $removed = 0;
my $errmsg = "";
my @src = ();
my @dst = ();

my $ajax_trigger = 0;
my $ajax_table_trigger = 0;
my $ajax_trigger = $cgi->param("ajax_trigger");
my $ajax_table_trigger = $cgi->param("ajax_table_trigger");

my %classactions;
my %shaperatess;

my $errmsg_tmp = '';

if ($removeedit ne '') {
	$remove = $removeedit;
}

if ($policy ne '') {
	$update = 1;
}
if (($policy < 1) || ($policy > 99 )) {
	$policy = 101;
}
if ($srcnet eq '') {
	$srcnet = "0.0.0.0/0";
}
if ($dstnet eq '') {
	$dstnet = "0.0.0.0/0";
}

# Skip rules if the values are suspect
if ($update == 1)
{
	@src = split(/\//, $srcnet);
	@dst = split(/\//, $dstnet);
	my $ipno = qr/2(?:5[0-5]|[0-4]\d)|1\d\d|[1-9]?\d/x;
	if ($src[0] !~ /^($ipno\.){3}$ipno$/) {
		$update = 0;
		$paramerror = 1;
	}
	if ($dst[0] !~ /^($ipno\.){3}$ipno$/) {
		$update = 0;
		$paramerror = 2;
	}
	if (!looks_like_number($src[1])) {
		$update = 0;
		$paramerror = 3;
	}
	if (!looks_like_number($dst[1])) {
		$update = 0;
		$paramerror = 3;
	}
	if (( $src[1] > 32) || ( $src[1] < 0 ) || ( $dst[1] > 32) || ( $dst[1] < 0 )) {
		$update = 0;
		$paramerror = 3;
	}
	if (($proto eq '') && ($port ne '')) {
		$update = 0;
		$paramerror = 4;
	}
	# if (($proto =~ /src/) || ($proto =~ /dst/) && ($port eq '')) { $update = 0; $paramerror = 4;}
	if (($proto ne '') && ($port ne ''))
	{
		if (( $port < 0 ) || ($port > 65535)) {
			$update = 0;
			$paramerror = 5;
		}
	}
	if (($port ne '') && (!looks_like_number($port))) {
		$update = 0;
		$paramerror = 5;
	}
	if (($rate ne '') && (!looks_like_number($rate))) {
		$update = 0;
		$paramerror = 6;
	}
	if ($rate < 8) {
		$rate = 8;
	}
	if ($rate > 1000000) {
		$rate = 1000000;
	}
	if ($action ne 'shape') {
		$rate = "-";
	}
	if (($action eq 'shape') && ($rate eq '')) {
		$update = 0;
		$paramerror = 7;
	}

}

my @rows;
my @index = ( 1 .. 99 );
open(ROW, "<", "/tce/etc/wanos/policies") || die("Couldn't open the file. Error: $!\n");
while (<ROW>)
{
	chomp;
	next if /^\s*\#/;
	next unless /=/;
	my ($key, $commastring) = split(/\s*=\s*/, $_, 2);
	$commastring =~ s/(\$(\w+))/$rows{$2}/g;
	$rows{$key} = $commastring;
	@index = grep {
		$_ != $key
	} @index;
}
close ROW;

open(GROUPOBJ, "<", "/tce/etc/wanos/groupobjects") || die("Couldn't open the file. Error: $!\n");
while (<GROUPOBJ>)
{
	chomp;
	next if /^\s*\#/;
	next unless /=/;
	my ($key, $commastring) = split(/\s*=\s*/, $_, 2);
	$commastring =~ s/(\$(\w+))/$groupobj{$2}/g;
	$groupobj{$key} = $commastring;
}
close GROUPOBJ;

if ($update == 1)
{
	foreach my $check (keys %rows)
	{
		if (($check == $policy) && ($check != $remove)) {
			$update = 0;
			$paramerror = 8;
		}

		## Is the QoS action consistent with this class
		my @values = split(',', $rows{$check});
		if ($values[4] eq $class)
		{
			if ($values[5] ne $action)
			{
				$update = 0;
				$paramerror = 9;
				$errmsg = "New QoS action $action is not consistent with the previously defined action $values[5] for class $class.<br>\nWhen policies are grouped together in the same class, they share a single common action.\n";
			}
			if (($values[5] eq $action) && ($values[6] ne $rate))
			{
				$update = 0;
				$paramerror = 9;
				$errmsg = "New Shape Rate $rate is not consistent with the previously defined shape rate $values[6] for class $class.<br>\nWhen policies are grouped together in the same class, they share a single common action.";
			}
		}
	}
}

if (($remove ne '') && (looks_like_number($remove)) && $paramerror == 0)
{
	open(OUT, ">", "/tce/etc/wanos/policies") || die("Couldn't open the file. Error: $!\n");
	foreach my $num (keys %rows)
	{
		$line = $num . "=" . "$rows{$num}" . "\n";
		print OUT $line unless ( $line =~ m/^$remove=/ );
	}
	close(OUT);
	$removed = $remove;

	open(OUT, ">", "/tce/etc/wanos/reconfigure") || die("Couldn't open the file. Error: $!\n");
	print OUT "1";
	close OUT;

}

if (( $removed > 0) && ( $update != 1 )) {
	print $cgi->header('text/plain;charset=UTF-8');
	print <<END
		<div class="alert alert-success alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Done!</strong>

END
;
	print "Rule # " . $removed . " removed. Apply the new policy by restarting the Wanos service under
		<a target='_parent' href='#/reset'>Configure > Reset</a><br />\n";
	print <<END
		</div>

END
}


if ($update == 1)
{
	open(OUT, ">>", "/tce/etc/wanos/policies") || die("Couldn't open the file. Error: $!\n");
	if ($port ne '') {
		$port = "port $port";
	}
	my $updatestring = "$policy" . "=" . "$src[0]/$src[1],$dst[0]/$dst[1],$proto $port,$dscp,$class,$action,$rate,$bypass";
	print OUT $updatestring . "\n";
	($key, $commastring) = split(/\s*=\s*/, $updatestring, 2);
	$commastring =~ s/(\$(\w+))/$rows{$2}/g;
	$rows{$key} = $commastring;
	@index = grep {
		$_ != $key
	} @index;
	close OUT;

	open(OUT, ">", "/tce/etc/wanos/reconfigure") || die("Couldn't open the file. Error: $!\n");
	print OUT "1";
	close OUT;
}

if ($ajax_trigger == 0) {

	if ($ajax_table_trigger == 0) {

		print <<END;

<div class="page-title">
    <div class="title_left">
        <h3>Network Traffic Policies</h3>
    </div>
</div>
<div class="clearfix"></div>
END
		;
	}

		if ($ajax_table_trigger == 0) {

		print <<EOF;

<div class="row">
    <div class="col-md-12 col-sm-12 col-xs-12">
        <div class="x_title title_block">
            <h2>Traffic Policies List</h2>
            <ul class="nav navbar-right panel_toolbox">
                <li><a class="collapse-link"><i class="fa fa-chevron-up"></i></a>
                </li>
                <li><a class="close-link"><i class="fa fa-close"></i></a>
                </li>
            </ul>
            <div class="clearfix"></div>
        </div>
        <div class="x_panel fixed_height_320">
            <div class="x_content">
EOF
		;
	}
	print <<EOF
				<table id="policymap_table" class="table table-striped">
				    <thead>
				        <tr>
				            <th width='5%'>
				                #
				            </th>
				            <th width='15%' title='Source Network (From Lan)'>
				                Source
				            </th>
				            <th width='15%' title='Destination Network (To Wan)'>
				                Destination
				            </th>
				            <th width='15%' title='Protocol and Port number'>
				                Port
				            </th>
				            <th width='8%' title='DSCP Value'>
				                DSCP
				            </th>
				            <th width='8%' title='Group policies together to perform a common QoS action'>
				                Class
				            </th>
				            <th width='8%' title='Set QoS action to apply to traffic.&#013;Default applies normal priority.&#013;Use high priority for low latency.&#013;Use low priority for best effort.&#013;Each class group has one action'>
				                QoS
				            </th>
				            <th width='8%' title='Rate in Kilobits per second (kbps)'>
				                Rate
				            </th>
				            <th width='8%' title='Enable pass-through without optimization'>
				                Bypass
				            </th>
				            <th width='6%'>

				            </th>
				        </tr>
				    </thead>

				    <tr>
				        <td>
				            <center></center>
				        </td>
				        <td title='Source Network (From Lan)'>0.0.0.0/0</td>
				        <td title='Destination Network (To Wan)'>0.0.0.0/0</td>
				        <td title='TCP: $groupobj{interactive}'>Interactive</td>
				        <td title='DSCP Value'>-</td>
				        <td title='Group policies together to perform a common QoS action'>17</td>
				        <td title='Set QoS action to apply to traffic.&#013;Default applies normal priority.&#013;Use high priority for low latency.&#013;Use low priority for best effort.&#013;Each class group has one action'>high</td>
				        <td title='Rate in Kilobits per second (kbps)'>-</td>
				        <td title='Enable pass-through without optimization'>
				            <center>&#10004;</center>
				        </td>
				        <td> </td>
				    </tr>

				    <tr>
				        <td>
				            <center></center>
				        </td>
				        <td title='Source Network (From Lan)'>0.0.0.0/0</td>
				        <td title='Destination Network (To Wan)'>0.0.0.0/0</td>
				        <td title='TCP: $groupobj{encrypted}'>Encrypted</td>
				        <td title='DSCP Value'>-</td>
				        <td title='Group policies together to perform a common QoS action'>0</td>
				        <td title='Set QoS action to apply to traffic.&#013;Default applies normal priority.&#013;Use high priority for low latency.&#013;Use low priority for best effort.&#013;Each class group has one action'>-</td>
				        <td title='Rate in Kilobits per second (kbps)'>-</td>
				        <td title='Enable pass-through without optimization'>
				            <center>&#10004;</center>
				        </td>
				        <td> </td>
				    </tr>
EOF
;
	foreach (sort {
		$a <=> $b
	} keys(%rows))
	{

		if ($removeedit ne $policy) {
			next if $_ == $removed;
		}
		my @values = split(',', $rows{$_});
		print "<tr data-id='$_'><td><center>$_</center></td><td title='Source Network (From Lan)'>$values[0]</td><td title='Destination Network (To Wan)'>$values[1]</td>\n";
		print "<td title='Protocol and Port number'>$values[2]</td><td title='DSCP Value'>$values[3]</td><td title='Group policies together to perform a common QoS action'>$values[4]</td>\n";
		print "<td title='Set QoS action to apply to traffic.&#013;Default applies normal priority.&#013;Use high priority for low latency.&#013;Use low priority for best effort.&#013;Each class group has one action'>$values[5]</td>\n";
		print "<td title='Rate in Kilobits per second (kbps)'>$values[6]</td>\n";
		print "<td title='Enable pass-through without optimization'><center>";
		if ($values[7] eq 'true') {
			print "&#10004;";
		}
		print "</center></td>\n";
		if (($_ <= 99) && ($_ >= 1)) {
			print  "<td><a class='btn_edit' href='#' data-id='$_' title='Edit this rule'><i class='fa fa-edit'></i></a>&nbsp;<span style='color:#ccc;'>|</span>&nbsp;<a class='btn_trash' href='#' data-id='$_' title='Remove this rule' ><i class='fa fa-trash-o'></i></a></td></tr>\n";
		}
		else {
			print "<td> </td></tr>\n";
		}

		$classactions{$values[4]} = $values[5];
		$shaperates{$values[4]} = $values[6];

	}
	print "</table>";

	if (1) {
		print <<END

            </div>
        </div>
    </div>
</div>
END
	;
	}

	if ($ajax_table_trigger == 0) {
		print <<EOF;


<div class="row">
    <div class="col-md-6 col-sm-6 col-xs-12">
        <div class="x_title title_block">
            <h2>Add Rule</h2>
            <ul class="nav navbar-right panel_toolbox">
                <li><a class="collapse-link"><i class="fa fa-chevron-up"></i></a>
                </li>
                <li><a class="close-link"><i class="fa fa-close"></i></a>
                </li>
            </ul>
            <div class="clearfix"></div>
        </div>
        <div class="x_panel fixed_height_320">
            <div class="x_content" id="add_rule">

EOF
		;
	}
}

my $i = $index[0];
my @editrule;
if ($edit ne '')
{
	$i = $edit;
	@editrule = split(',', $rows{$edit});
}

if ($update == 1) {

	print <<END
		<div class="alert alert-success alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Done!</strong> Policy updated. Apply the new policy by restarting the Wanos service under <a target='_parent' href='#/reset'>Configure > Reset</a><br />\n <br />
		</div>

END
;
}
if ($paramerror == 1) {
	$errmsg = "Invalid Source Network<br />\n";
}
if ($paramerror == 2) {
	$errmsg = $errmsg . "Invalid Destination Network<br />\n";
}
if ($paramerror == 3) {
	$errmsg = $errmsg .  "Invalid Network Mask<br />\n";
}
if ($paramerror == 4) {
	$errmsg = $errmsg .  "Invalid Protocol and Port combination<br />\n";
}
if ($paramerror == 5) {
	$errmsg = $errmsg .  "Invalid Port Number<br />\n";
}
if ($paramerror == 6) {
	$errmsg = $errmsg .  "Invalid Shaping Rate Number in Kilobits per second<br />\n";
}
if ($paramerror == 7) {
	$errmsg = $errmsg .  "Shape requires a Rate<br />\n";
}
if ($paramerror == 8) {
	$errmsg = $errmsg .  "Duplicate Rule Number<br />\n";
}

if ($errmsg ne '') {
	print <<END
		<div class="alert alert-danger alert-dismissible fade in" role="alert">
		    <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
		    <strong>Error!</strong> $errmsg
		</div>

END
;
}

if ($ajax_table_trigger == 0) {
	print <<EOF;

				<form id="traffic_policies" class="form-horizontal form-label-left" name="addrule" action="" method="post" data-parsley-validate>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="policy">Rule #
                        </label>
                        <div class="col-md-4 col-sm-4 col-xs-12">
                            <input id="policy" name="policy" title="Rule Number (1-99)"
		                        type="text" value="$i" min="1" max="99" size="1" data-parsley-range="[1, 99]"
		                        class="form-control col-md-12 col-lm-12 col-xs-12" data-parsley-required
		                        data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="srcnet">Source <span class="description">Ex: 192.168.0.1/24</span></label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="srcnet" title="Traffic Source Network & Mask"
                            type="text" size="14" maxlength="18"
                            id="srcnet" class="form-control col-md-4 col-xs-12 " data-parsley-check_ip_and_mask,
                            data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="dstnet">Destination <span class="description">Ex: 192.168.0.1/24</span></label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="dstnet" title="Traffic Destination Network & Mask"
                            type="text" size="14" maxlength="18"
                            id="dstnet" class="form-control col-md-4 col-xs-12 " data-parsley-check_ip_and_mask
                            data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="proto">Port</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <select id="proto" name="proto" title="TCP or UDP Protocol" onclick="enableport()" id="proto" class="form-control" data-toggle="tooltip">
		                        <option value="tcp">tcp</option>
		                        <option value="udp">udp</option>
		                        <option value="src tcp">src tcp</option>
		                        <option value="dst tcp">dst tcp</option>
		                        <option value="src udp">src udp</option>
		                        <option value="dst udp">dst udp</option>
		                    </select>
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="port">Port number #</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="port" type="text" title="TCP or UDP port number"
                            size="2" id="port" class="form-control col-md-4 col-xs-12 "
                            data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="dscp">DSCP</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <select id="dscp" name="dscp" title="Set the DSCP value. Default is to preserve existing markings" id="dscp" class="form-control" data-toggle="tooltip">
		                        <option value="-">-</option>
		                        <option value="CS0">CS0</option>
		                        <option value="CS1">CS1</option>
		                        <option value="AF11">AF11</option>
		                        <option value="AF12">AF12</option>
		                        <option value="AF13">AF13</option>
		                        <option value="CS2">CS2</option>
		                        <option value="AF21">AF21</option>
		                        <option value="AF22">AF22</option>
		                        <option value="AF23">AF23</option>
		                        <option value="CS3">CS3</option>
		                        <option value="AF31">AF31</option>
		                        <option value="AF32">AF32</option>
		                        <option value="AF33">AF33</option>
		                        <option value="CS4">CS4</option>
		                        <option value="AF41">AF41</option>
		                        <option value="AF42">AF42</option>
		                        <option value="AF43">AF43</option>
		                        <option value="CS5">CS5</option>
		                        <option value="EF">EF</option>
		                        <option value="CS6">CS6</option>
		                        <option value="CS7">CS7</option>
		                    </select>
                        </div>
                    </div>


                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="class">Class</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <select name="class" title="Group policies together to perform a common QoS action" id="class" onchange="checkactions(this.value)" class="form-control" data-toggle="tooltip">
		                        <option value="0">-</option>
		                        <option value="1">1</option>
		                        <option value="2">2</option>
		                        <option value="3">3</option>
		                        <option value="4">4</option>
		                        <option value="5">5</option>
		                        <option value="6">6</option>
		                        <option value="7">7</option>
		                        <option value="8">8</option>
		                        <option value="9">9</option>
		                        <option value="10">10</option>
		                        <option value="11">11</option>
		                        <option value="12">12</option>
		                        <option value="13">13</option>
		                        <option value="14">14</option>
		                        <option value="15">15</option>
		                        <option value="16">16</option>
		                    </select>
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="action">QoS</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <select id="action" name="action" title="Set QoS action to apply to traffic.&#013;Default applies normal priority.&#013;Use high priority for low latency.&#013;Use low priority for best effort.&#013;Each class group has one action" onchange="enablerate()" class="form-control" data-toggle="tooltip">
		                        <option value="-">Default</option>
		                        <option value="high">High Priority</option>
		                        <option value="low">Low Priority</option>
		                        <option value="shape">Shape</option>
		                        <option value="drop">Drop</option>
		                    </select>
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="rate">Rate</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input name="rate" disabled
                            title="Rate in Kilobits per second (kbps).&#013;One Rate per Shaper."
                            type="text" size="3"
                            id="rate" class="form-control" data-parsley-type="digits"
                            data-toggle="tooltip" />
                        </div>
                    </div>

                    <div class="form-group">
                        <label class="control-label col-md-3 col-sm-3 col-xs-12" for="bypass">Bypass</label>
                        <div class="col-md-6 col-sm-6 col-xs-12">
                            <input type="checkbox" name="bypass"
                            title="Enable pass-through without optimization"
                            value="true"
                            id="bypass" class="form-control flat"
                            data-toggle="tooltip" />
                        </div>
                    </div>


                    <div class="ln_solid"></div>
                    <div class="form-group">
                        <div class="col-md-6 col-sm-6 col-xs-12 col-md-offset-3">
                            <a type="button" class="btn btn-default" href="/#/system_settings" onclick="window.location.reload();return false;" style='font-size: 8pt;'>Cancel</a>
                            <button id="btn_submit" type="submit" class="btn btn-success" style='font-size: 8pt;'>Submit</button>
                        </div>
                    </div>

EOF

	if ($edit ne '')
	{
		($editproto, $editport) = split('port', $editrule[2]);
		chop($editproto);
		print "<input id='removeedit' type='hidden' name='removeedit' value='$edit'>\n";
		print <<EOF;
	<script>
	document.getElementById("srcnet").value = "$editrule[0]";
	document.getElementById("dstnet").value = "$editrule[1]";
	document.getElementById("proto").value = "$editproto";
	document.getElementById("port").value = "$editport";
	document.getElementById("dscp").value = "$editrule[3]";
	document.getElementById("class").value = "$editrule[4]";
	document.getElementById("action").value = "$editrule[5]";
	document.getElementById("bypass").checked = $editrule[7];
	var checkbox = "$editrule[6]";
	if( checkbox == "-" ){
	document.getElementById("rate").disabled = true;
	 }
	else {
	document.getElementById("rate").value = "$editrule[6]";
	document.getElementById("rate").disabled = false;
	document.getElementById("rate").readOnly= true;
	}

	</script>
</form>

EOF
	}
}
if ($ajax_trigger == 0) {

	print <<EOF;
	<script type="text/javascript" langage="JavaScript">
	function checkactions(classselected) {
	var classaction = [];
EOF

	foreach (sort {
		$a <=> $b
	} keys(%classactions))
	{
		print "classaction[\"$_\"] = \"$classactions{$_}\"\n";
	}


	print " var shaperate = [];\n";
	foreach (sort {
		$a <=> $b
	} keys(%shaperates))
	{
		print "shaperate[\"$_\"] = \"$shaperates{$_}\"\n";
	}

	print <<EOF;

	var op = document.getElementById("action").getElementsByTagName("option");
	if (classaction[classselected])
		{
		document.getElementById("action").value = classaction[classselected];

		for (var i = 0; i < op.length; i++)
			{
		  	(op[i].value.toLowerCase() == classaction[classselected])
		    	? op[i].disabled = false
		    	: op[i].disabled = true ;
			}
		}
		else
		{
		document.getElementById("action").value = "-";
		for (var i = 0; i < op.length; i++)
			{
			op[i].disabled = false;
			}
		}

	if ( classaction[classselected] == "shape" )
		{
		  document.getElementById("rate").value = shaperate[classselected];
		  document.getElementById("rate").disabled = false;
        	  document.getElementById("rate").readOnly= true;
		}
	else { document.getElementById("rate").value = ""; }

	}

	var classvalue = document.getElementById("class").value;
	checkactions(classvalue);

function enablerate() {
var val = document.addrule.action.value;
if(val == "shape")
{
document.getElementById("rate").readOnly= false;
document.getElementById("rate").disabled = false;
}
else { document.getElementById("rate").disabled = true; }
}

	</script>
EOF

}

if ($ajax_trigger == 0) {

	print <<END
			</div>
		</div>
	</div>
</div>

END
;

}
